@@ -1,5 +1,7 @@
 # -*- coding: utf-8 -*-
 import math
+import os
+from os import path
 import traceback
 from typing import Union
 from copy import deepcopy
@@ -11,6 +13,7 @@ from colorama import Fore
 from ..lexer import Lexer, default
 from .. import parser
 from ..ast import expressions
+from .. import STD, PKGS
 
 
 class Stmt(AnyBase):
@@ -620,6 +623,7 @@ class FuncStmt(Stmt):
         self.name = name
         self.args = args
         self.body = body
+        self.decorators = None
 
     def __repr__(self) -> str:
         return f"FuncStmt({self.name}, {self.args}, {self.body})"
@@ -629,7 +633,45 @@ class FuncStmt(Stmt):
         if has_var and not is_const and level == lvl:
             signal.ERROR = f"Function {self.name} is exists"
             return
-        env[lvl][self.name] = (self.args, self.body)
+        env[lvl][self.name] = (self.args, self.body, self.decorators, self.name)
+
+
+class DecoratorStmt(Stmt):
+    def __init__(self, names, function):
+        self.names = names
+        self.function = function
+
+    def __repr__(self) -> str:
+        return f"DecoratorStmt({self.names}, {self.function})"
+
+    def eval(self, env, consts, lvl, modules, signal):
+        for name in self.names:
+            obj = None
+            if isinstance(name, str):
+                obj = expressions.VarAST(name).eval(env, consts, lvl, modules, signal)
+            elif isinstance(name, (expressions.ClassPropAST, expressions.ModuleCallAST)):
+                obj = name.eval(env, consts, lvl, modules, signal)
+            elif isinstance(name, CallStmt):
+                obj = (name.name, name.args)
+            if obj is not None:
+                if isinstance(obj, tuple):
+                    if self.function.decorators is None:
+                        self.function.decorators = [{
+                            'name': name,
+                            'func_name': self.function.name
+                        }]
+                    else:
+                        self.function.decorators.append({
+                            'name': name,
+                            'func_name': self.function.name
+                        })
+                else:
+                    signal.ERROR = f"{name} is not function"
+                    return
+            else:
+                signal.ERROR = f"{name} is not assigned"
+                return
+        self.function.eval(env, consts, lvl, modules, signal)
 
 
 class LambdaStmt(Stmt):
@@ -641,7 +683,7 @@ class LambdaStmt(Stmt):
         return f"LambdaStmt({self.args}, {self.body})"
 
     def eval(self, env, consts, lvl, modules, signal):
-        return self.args, self.body
+        return self.args, self.body, None, None
 
 
 class CallStmt(Stmt):
@@ -677,6 +719,29 @@ class CallStmt(Stmt):
             signal.IN_CLASS = True
             f = self.name.eval(env, consts, lvl, modules, signal)
         if f is not None:
+            if len(f) == 4 and f[2] is not None:
+                index = 0
+                for i in f[2][::-1]:
+                    if i['func_name'] == f[3] and f[3] != self.name:
+                        break
+                    if index > 0:
+                        func = f
+                        if isinstance(func, tuple):
+                            func = LambdaStmt(f[0], f[1])
+                        else:
+                            signal.ERROR = f"is not function"
+                    else:
+                        func = expressions.VarAST(i['func_name'])
+                    if isinstance(i['name'], CallStmt):
+                        i['name'].args.insert(0, expressions.ArgumentAST(None, func))
+                        f = i['name'].eval(env, consts, lvl, modules, signal)
+                        i['name'].args.pop(0)
+                    elif isinstance(i['name'], str):
+                        call = CallStmt(
+                            i['name'], [expressions.ArgumentAST(None, func)]
+                        )
+                        f = call.eval(env, consts, lvl, modules, signal)
+                    index += 1
             args = [i for i in self.args if i.name is None]
             fargs = [i for i in f[0] if i.value is None]
             kwargs = [i for i in self.args if i.name is not None]
@@ -742,6 +807,12 @@ class ReturnStmt(Stmt):
 
 
 class ImportStmt(Stmt):
+    PATHS = [
+        path.curdir,
+        STD,
+        PKGS
+    ]
+
     def __init__(self, module_name, objects, from_import):
         self.module_name = module_name
         self.objects = objects
@@ -750,13 +821,28 @@ class ImportStmt(Stmt):
     def __repr__(self) -> str:
         return f"ImportStmt({self.module_name}, {self.objects}, {self.from_import})"
 
+    @staticmethod
+    def _find(file) -> bool:
+        return exists(file) and isfile(file)
+
     def eval(self, env, consts, lvl, modules, signal):
         current_module = signal.CURRENT_MODULE
+        current_dir = os.getcwd()
         if self.module_name is not None:
-            module_name = self.module_name + '.avo'
-            if not exists(module_name) or not isfile(module_name):
-                signal.ERROR = f"can't find module {module_name}"
+            module_name = None
+            for i in self.PATHS:
+                # Find module.avo or module/init.avo
+                module_name = path.join(i, self.module_name + '.avo')
+                if self._find(module_name):
+                    break
+                module_name = path.join(i, self.module_name, 'init.avo')
+                if self._find(module_name):
+                    break
+            if module_name is None:
+                signal.ERROR = f"{self.module_name} isn't exists"
                 return
+            # Change current dir to module dir
+            os.chdir(path.dirname(path.abspath(module_name)))
             statements = parser.stmt_list()(Lexer.lex_file(module_name), 0)
             if statements:
                 signal.CURRENT_MODULE = module_name
@@ -774,6 +860,7 @@ class ImportStmt(Stmt):
                 self.module_name = self.objects.pop(0)
                 self.eval(env, consts, lvl, modules, signal)
         signal.CURRENT_MODULE = current_module
+        os.chdir(current_dir)
 
 
 class EOFStmt(Stmt):
